# -*- coding: utf-8 -*-

"""
Serializer Mixins
=================

Per rendere più strutturati i nostri Serializer, per renderli più omogenee
e per inserire validazioni comuni in base alla tipologia di vista a cui sono associati, sono
stati creati dei mixin con metodo validate (vedi in particolare
:class:`ThuxCreateSerializerMixin`,  :class:`ThuxUpdateSerializerMixin`,
:class:`ThuxStatusSerializerMixin`, :class:`ThuxEnableSerializerMixin` e :class:`ThuxDisableSerializerMixin`),
oppure solo come superclassi per un utilizzo futuro:

* :class:`PermissionSerializer`
* :class:`ThuxListSerializerMixin`
* :class:`ThuxRetrieveSerializerMixin`
* :class:`ThuxCreateSerializerMixin`
* :class:`ThuxUpdateSerializerMixin`
* :class:`ThuxPartialUpdateSerializerMixin`
* :class:`ThuxStatusSerializerMixin`
* :class:`ThuxEnableSerializerMixin`
* :class:`ThuxDisableSerializerMixin`

Esempio
--------

::

  from thx-api.serializers import (
        ThuxListSerializerMixin, ThuxRetrieveSerializerMixin, ThuxCreateSerializerMixin,
        ThuxUpdateSerializerMixin, ThuxPartialUpdateSerializerMixin,
        ThuxDisableSerializerMixin, ThuxEnableSerializerMixin,
  )

  class TemplateUpdateSerializer(ThuxUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        model = models.Template
        exclude = (
            'date_create', 'date_last_modify', 'creator', 'last_modifier'
        )

Classi
========

.. autoclass:: PermissionSerializer
   :members:

.. autoclass:: ThuxListSerializerMixin
   :members:

.. autoclass:: ThuxRetrieveSerializerMixin
   :members:

.. autoclass:: ThuxCreateSerializerMixin
   :members:

.. autoclass:: ThuxUpdateSerializerMixin
   :members:

.. autoclass:: ThuxPartialUpdateSerializerMixin
   :members:

.. autoclass:: ThuxStatusSerializerMixin
   :members:

.. autoclass:: ThuxEnableSerializerMixin
   :members:

.. autoclass:: ThuxDisableSerializerMixin
   :members:

"""

import copy

from django.contrib.auth import models
from django.core.exceptions import ValidationError

from rest_framework import serializers, exceptions
from rest_framework.utils import model_meta


def clean_instance(attrs, instance, info):
    # TODO da commentare
    many_to_many = {}
    # TODO: verificare se copy è necessario
    attr_copy = copy.copy(attrs)
    for attr, value in attr_copy.items():
        if info.relations.get(attr) and info.relations.get(attr).to_many:
            many_to_many[attr] = attrs.pop(attr)
        else:
            setattr(instance, attr, value)
    try:
        instance.clean()
        attrs.update(many_to_many)
    except ValidationError as ve:
        if hasattr(ve, 'error_dict'):
            raise exceptions.ValidationError(detail=ve.message_dict)
        raise exceptions.ValidationError(detail=ve.messages)
    return attrs


class PermissionSerializer(serializers.ModelSerializer):
    """
    **PermissionSerializer** serializer base per vista :class:`PermissionListView`
    """

    class Meta:
        model = models.Permission
        fields = '__all__'


class ThuxListSerializerMixin:
    """
    **ThuxListSerializerMixin** è un mixin vuoto predisposto per un utilizzo futuro
    """

    pass


class ThuxRetrieveSerializerMixin:
    """
    **ThuxRetrieveSerializerMixin** è un mixin vuoto predisposto per un utilizzo futuro
    """

    pass


class ThuxCreateSerializerMixin:
    # TODO da commentare
    def validate(self, attrs):
        super().validate(attrs)
        _attrs = copy.copy(attrs)
        info = model_meta.get_field_info(self.Meta.model)
        many_to_many = {}
        for attr, value in _attrs.items():
            if info.relations.get(attr) and info.relations.get(attr).to_many:
                many_to_many[attr] = attrs.pop(attr)

        attrs["creator"] = self.context["request"].user
        attrs["last_modifier"] = self.context["request"].user
        instance = self.Meta.model(**attrs)
        try:
            instance.clean()
            attrs.update(many_to_many)
        except ValidationError as ve:
            if hasattr(ve, 'error_dict'):
                raise exceptions.ValidationError(detail=ve.message_dict)
            raise exceptions.ValidationError(detail=ve.messages)
        return attrs


class ThuxUpdateSerializerMixin:
    # TODO da commentare
    def validate(self, attrs):
        super().validate(attrs)
        info = model_meta.get_field_info(self.Meta.model)
        attrs["last_modifier"] = self.context["request"].user
        return clean_instance(attrs, self.instance, info)


class ThuxPartialUpdateSerializerMixin(ThuxUpdateSerializerMixin):
    """
    **ThuxPartialUpdateSerializerMixin** è un mixin vuoto predisposto per un utilizzo futuro
    """

    pass


class ThuxStatusSerializerMixin:
    # TODO da commentare
    def validate(self, attrs):
        super().validate(attrs)
        info = model_meta.get_field_info(self.Meta.model)
        status = attrs.get("status")
        attrs.clear()
        attrs["last_modifier"] = self.context["request"].user
        attrs["status"] = status
        return clean_instance(attrs, self.instance, info)


class ThuxEnableSerializerMixin:
    # TODO da commentare
    def validate(self, attrs):
        super().validate(attrs)
        info = model_meta.get_field_info(self.Meta.model)
        attrs["last_modifier"] = self.context["request"].user
        attrs["status"] = 1
        return clean_instance(attrs, self.instance, info)


class ThuxDisableSerializerMixin:
    # TODO da commentare
    def validate(self, attrs):
        super().validate(attrs)
        info = model_meta.get_field_info(self.Meta.model)
        attrs["last_modifier"] = self.context["request"].user
        attrs["status"] = 0
        return clean_instance(attrs, self.instance, info)
